#!/usr/bin/env python
import sliderepl
sliderepl.SADeck.run()

### slide:: 1

from sqlalchemy import Table, MetaData, Column, Integer, String
from sqlalchemy.orm import mapper, create_session, compile_mappers

metadata = MetaData('sqlite://')
users = Table('users', metadata,
              Column('id', Integer, primary_key=True),
              Column('name', String(50)))
metadata.create_all()

### slide:: 2

class User(object):
    pass
list(vars(User))

### slide:: 3

mapper(User, users)
compile_mappers()
list(vars(User))

### slide:: 4

# mapping twice is an error
mapper(User, users)
class User(object):
    pass

# using without mapping is an error
session = create_session()
session.query(User).all()

# restore the mapping we just broke
mapper(User, users)

### slide:: 5

u = User()
u.name is None

### slide:: 6
u.name is None
User.name is None
type(User.name)

### slide:: 7

u = User()
'name' in u.__dict__
u.name
'name' in u.__dict__

### slide:: 8

u = User()

### slide:: 9

session.save(u)
session.flush()
u.id

### slide:: 10

u = User()
vars(u)

### slide:: 11

session.save(u)
vars(u)

### slide:: 12

session.flush()
vars(u)


### slide:: 13

from sqlalchemy import ForeignKey
from sqlalchemy.orm import relation

addresses = Table('addresses', metadata,
    Column('id', Integer, primary_key=True),
    Column('email', String(50)),
    Column('user_id', Integer, ForeignKey('users.id'))
)
metadata.create_all()

class User(object):
    pass
class Address(object):
    def __init__(self, email=None):
        self.email = email
    def __eq__(self, other):
        return self.email == other.email
mapper(User, users, properties={
  'addresses': relation(Address, backref='user')})
mapper(Address, addresses)

### slide:: 14

u = User()
a = Address('fu@foo.foo')
u.addresses.append(a)

session.save(u)
session.flush()

# There's a user that contains a
session.query(User).filter(User.addresses.contains(a)).all()

this_address = Address('fu@foo.foo')
this_address == a
# But not one with this_address
session.query(User).filter(User.addresses.contains(this_address)).all()

### slide:: 15

u = User()
a = Address()
a.user = u
u.addresses

### slide:: end
